// Engine.cpp: implementation of the CEngine class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Engine.h"

#define PI  3.1415926535897932384626433832795
#define EPS 0.001


CObj::CObj(COLORREF color_,int x0,int y0,int r0,double dx0,double dy0)
{
	m=r0*r0;
	color=color_;
	x=x0; y=y0; r=r0; dx=dx0; dy=dy0;
}


bool CObj::isStand()
{
	return fabs(dx)<EPS && fabs(dy)<EPS;
}

void CEngine::Init(CRect &rect_)
{
	int h=(int)(sin(PI/3)*20);
	rect=rect_; 
	m_bar.SetRect(90,170,240,190);	
	m_list.push_back(CObj(RGB(-1,-1,-1  ),30,20,16,3.7,3.1));
	m_list.push_back(CObj(RGB(-1,-1,-90 ),200,96,14));
	m_list.push_back(CObj(RGB(-1,-90,-1 ),200+10,100+h  ));
	m_list.push_back(CObj(RGB(-90,-1,-1 ),200-10,100+h  ));
	m_list.push_back(CObj(RGB(-1,-90,-90),200-20,100+h*2));
	m_list.push_back(CObj(RGB(-90,-90,-1),200   ,100+h*2));
	m_list.push_back(CObj(RGB(-90,-1,-90),200+20,100+h*2));
}

const int CEngine::GetCount()
{
	return m_list.size();
}

const CObj *CEngine::GetObj(int index)
{
	return &m_list[index];
}


void CEngine::Work()
{
	for(unsigned int n=0; n<m_list.size(); n++) {
		CObj *obj=&m_list[n];
		obj->x+=obj->dx;
		obj->y+=obj->dy;
		CheckCollision(n,obj);
		CheckBar(obj);
		CheckBorder(obj);
	}
}


void CEngine::CheckBorder(CObj *obj)
{
	unsigned int f=0;
	if(obj->x+obj->r>rect.right ) f|=1; 
	if(obj->y+obj->r>rect.bottom) f|=2; 
	if(obj->x-obj->r<rect.left  ) f|=1;
	if(obj->y-obj->r<rect.top   ) f|=2;
	if(f&1)obj->x-=obj->dx,obj->dx=-obj->dx;
	if(f&2)obj->y-=obj->dy,obj->dy=-obj->dy;
}


void CEngine::CheckCollision(int index,CObj *obj1)
{
	for(unsigned int n=index+1; n<m_list.size(); n++) {
		CObj *obj2=&m_list[n];
		if(obj1->isStand() && obj2->isStand()) return; //     ,   
		double dx=obj2->x-obj1->x;
		double dy=obj2->y-obj1->y;
		double r=obj1->r+obj2->r;						//      
		double d=dx*dx+dy*dy;							//     
		if(d>=r*r) continue;
		d=sqrt(d);										//   
		double s=(r-d)/2;								//  
		double nx=dx/d;									//   
		double ny=dy/d;
		obj1->x-=s*nx; obj1->y-=s*ny;					//   
		obj2->x+=s*nx; obj2->y+=s*ny;
		double m1=1/obj1->m;
		double m2=1/obj2->m;
		double v1=obj1->dx*nx+obj1->dy*ny;				//     
		double v2=obj2->dx*nx+obj2->dy*ny;
		double f=2*(v1-v2)/(m1+m2);
		obj1->dx-=nx*f*m1; obj1->dy-=ny*f*m1;			//     
		obj2->dx+=nx*f*m2; obj2->dy+=ny*f*m2; 
	}
}


double CEngine::Distance2(CPoint &point,CObj *obj)
{
	double dx=point.x-obj->x;
	double dy=point.y-obj->y;
	return dx*dx+dy*dy;
}


void CEngine::CheckBar(CObj *obj)
{
	double dx=m_bar.Width();
	double dy=m_bar.Height();
	double r=dx*dx+dy*dy+obj->r+obj->r;					//         
	double d1=Distance2(m_bar.TopLeft(),obj);			//    
	double d2=Distance2(m_bar.BottomRight(),obj);
	if(d1<r && d2<r) {		//      
		CPoint point;
		r=sqrt(r);
		d1=sqrt(d1);
		d2=sqrt(d2);
		if(d1>d2) {			//      
			point.x=m_bar.left+(m_bar.right-m_bar.left)*d1/r;
			point.y=m_bar.top+(m_bar.bottom-m_bar.top)*d1/r;
		}else{
			point.x=m_bar.right+(m_bar.left-m_bar.right)*d2/r;
			point.y=m_bar.bottom+(m_bar.top-m_bar.bottom)*d2/r;
		}
		CheckPoint(point,obj);
	}
	CheckPoint(m_bar.TopLeft(),obj);				//     
	CheckPoint(m_bar.BottomRight(),obj);
}

void CEngine::CheckPoint(CPoint &point,CObj *obj)
{
	double dx=point.x-obj->x;
	double dy=point.y-obj->y;
	double r=sqrt(dx*dx+dy*dy);
	if(r>obj->r) return;
	double d=obj->r-r;
	dx/=r; dy/=r;
	obj->x-=dx*d; obj->y-=dy*d;		//   
    double v=obj->dx*dx+obj->dy*dy;	//     
	obj->dx-=2*v*dx;				//      
	obj->dy-=2*v*dy;
}


const CRect &CEngine::GetBar()
{
	return m_bar;
}




